import matplotlib.pyplot as plt
import numpy as np

file_dft = 'o.bands_interpolated_dft'  # DFT bands data file
file_gw = 'o.bands_interpolated_gw'  # GW bands data file
file_out = 'bandstructure.pdf'  # Output file name
interactive = False  # Interactive or output as file
homo = 13  # HOMO band index used for shifting
shift_mode = 'homo'  # Shift mode: homo, lumo, half or none
shift = 0  # Additional shift to the bands
k_points = {  # Define x labels for special k-points
    (0.00000, 0.00000, 0.00000): '\Gamma',
    (0.50000, 0.00000, 0.00000): 'M',
    (0.33333, 0.33333, 0.00000): 'K',
}
# Specify the index of the k-point to draw a vertical line on (0-based),
# if negative number, don't draw a vertical line
vertical_index = -1
ylim = (-4, 4)  # y-axis limits
plt.rc('figure', **{'figsize': [4, 6], 'dpi': 300})

dft = np.loadtxt(file_dft)
gw = np.loadtxt(file_gw)
num_bands = len(dft[0]) - 4
x = gw[:, 0]
dft_homo = np.amax(dft[:, homo])
dft_lumo = np.amin(dft[:, homo + 1])
gw_homo = np.amax(gw[:, homo])
gw_lumo = np.amin(gw[:, homo + 1])

# Plot the bands
for ib in range(1, num_bands + 1):
    emf = dft[:, ib] + shift
    eqp = gw[:, ib] + shift

    if shift_mode == 'homo':
        emf -= dft_homo
        eqp -= gw_homo
    elif shift_mode == 'lumo':
        emf -= dft_lumo
        eqp -= gw_lumo
    elif shift_mode == 'half':
        emf -= (dft_homo + dft_lumo) / 2
        eqp -= (gw_homo + gw_lumo) / 2

    lmf, = plt.plot(x, emf, '--', color='#3f51b5', lw=1.0, zorder=2)
    lqp, = plt.plot(x, eqp, '-', color='#f44336', lw=1.0, zorder=1)

# Set x labels using special_points
indices = []
labels = []
for index, kx, ky, kz in dft[:, [0, -3, -2, -1]]:
    for k in k_points:
        if np.linalg.norm(np.array([kx, ky, kz]) - np.array(k)) < 1e-4:
            indices.append(index)
            labels.append(k_points[k])
plt.xticks(indices, map(lambda s: '$\mathregular{%s}$' % s, labels))
plt.ylabel('Energy (eV)')

plt.axhline(0, color='#dddddd', lw=1, zorder=0)
# Set x-axis vertical lines
if vertical_index >= 0:
    plt.axvline(indices[vertical_index], color='k', lw=1, zorder=3)

plt.legend([lmf, lqp],
           ['DFT', 'GW'],
           bbox_to_anchor=(1.0, 1.0),
           loc='upper right',
           frameon=True,
           borderaxespad=0.25)
plt.xlim(x[0], x[-1])
plt.ylim(ylim)
plt.tight_layout()

if interactive:
    plt.show()
else:
    plt.savefig(file_out)
